package org.processmininguc.configurableprocesses.plugins;

import java.util.Collection;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.processtree.ProcessTree;
import org.processmininguc.configurableprocesses.algorithms.ConfigurableProcessesResult;
import org.processmininguc.configurableprocesses.algorithms.GreedyAlgorithm;
import org.processmininguc.configurableprocesses.connections.GreedyConnection;
import org.processmininguc.configurableprocesses.dialogs.GreedyDialog;
import org.processmininguc.configurableprocesses.help.GreedyHelp;
import org.processmininguc.configurableprocesses.parameters.GreedyParameters;

@Plugin(name = "Greedy Configurable Process Tree Configuration", 
parameterLabels = { "Configurable Process Tree", "Log", "Parameters" }, 
	    returnLabels = { "Configuration Summary" , "Derived Process Tree - Greedy strategy" }, 
	    returnTypes = { ConfigurableProcessesResult.class, ProcessTree.class }, 
	    help = GreedyHelp.TEXT)

public class GreedyPlugin extends GreedyAlgorithm {

	/**
	 * The plug-in variant that runs in any context and requires a parameters.
	 * 
	 * @param context The context to run in.
	 * @param input1 The first input.
	 * @param input2 The second input.
	 * @param parameters The parameters to use.
	 * @return The output.
	 */
	@UITopiaVariant(affiliation = "Universidad Catolica de Chile", 
					author = "Mauricio Arriagada-Benitez", 
					email = "marriagb@uc.cl")
	@PluginVariant(variantLabel = "Greedy Configurable Process Tree Configuration, parameters", 
					requiredParameterLabels = { 0, 1, 2 })
	public Object[] run(PluginContext context, ProcessTree tree, XLog log, GreedyParameters parameters,
			final ConfigurableProcessesResult confProcResults) {
//		((UIPluginContext) context).getGlobalContext().getResourceManager().getResourceForInstance(confProcResults).setFavorite(true);
		// Apply the algorithm depending on whether a connection already exists.
		return runConnections(context, tree, log, parameters, confProcResults);

	}
	
	/**
	 * The plug-in variant that runs in any context and uses the default parameters.
	 * 
	 * @param context The context to run in.
	 * @param input1 The first input.
	 * @param input2 The second input.
	 * @return The output.
	 */
	@UITopiaVariant(affiliation = "Universidad Catolica de Chile", author = "Mauricio Arriagada-Benitez", email = "marriagb@uc.cl")
	@PluginVariant(variantLabel = "Greedy Configurable Process Tree Configuration, default", requiredParameterLabels = { 0, 1 })
	public Object[] runDefault(PluginContext context, ProcessTree tree, XLog log) {
		// Get the default parameters.
	    GreedyParameters parameters = new GreedyParameters(tree, log);
		// Apply the algorithm depending on whether a connection already exists.
	    final ConfigurableProcessesResult confProcResults=new ConfigurableProcessesResult();
		return runConnections(context, tree, log, parameters, confProcResults);
	}
	
	/**
	 * The plug-in variant that runs in a UI context and uses a dialog to get the parameters.
	 * 
	 * @param context The context to run in.
	 * @param input1 The first input.
	 * @param input2 The second input.
	 * @return The output.
	 */
	@UITopiaVariant(affiliation = "Universidad Catolica de Chile", author = "Mauricio Arriagada-Benitez", email = "marriagb@uc.cl")
	@PluginVariant(variantLabel = "Greedy Configurable Process Tree Configuration, dialog", requiredParameterLabels = { 0, 1 })
	public Object[] runUI(UIPluginContext context, ProcessTree tree, XLog log) {
		// Get the default parameters.
	    GreedyParameters parameters = new GreedyParameters(tree, log);
	    // Get a dialog for this parameters.
	    GreedyDialog dialog = new GreedyDialog(context, tree, log, parameters);
	    // Show the dialog. User can now change the parameters.
	    InteractionResult result = context.showWizard("Greedy Configurable Process Tree Configuration Settings", true, true, dialog);
	    // User has close the dialog.
	    if (result == InteractionResult.FINISHED) {
			// Apply the algorithm depending on whether a connection already exists.
		    final ConfigurableProcessesResult confProcResults=new ConfigurableProcessesResult();
			return runConnections(context, tree, log, parameters,confProcResults);
	    }
	    // Dialog got canceled.
	    return null;
	}		
	
	/**
	 * Apply the algorithm depending on whether a connection already exists.
	 * 
	 * @param context The context to run in.
	 * @param input1 The first input.
	 * @param input2 The second input.
	 * @return The output.
	 */
	private Object[] runConnections(PluginContext context, ProcessTree tree, XLog log, GreedyParameters parameters, 
			final ConfigurableProcessesResult confProcResults) {
		if (parameters.isTryConnections()) {
			// Try to found a connection that matches the inputs and the parameters.
			Collection<GreedyConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(
						GreedyConnection.class, context, tree, log);
				for (GreedyConnection connection : connections) {
					if (connection.getObjectWithRole(GreedyConnection.TREE)
							.equals(tree) && connection.getObjectWithRole(GreedyConnection.LOG)
							.equals(log) && connection.getParameters().equals(parameters)) {
						// Found a match. Return the associated output as result of the algorithm.
						return connection
								.getObjectWithRole(GreedyConnection.OUTPUT);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		// No connection found. Apply the algorithm to compute a fresh output result.
		ProcessTree outputTree = apply(context, tree, log, parameters, confProcResults);
		if (parameters.isTryConnections()) {
			// Store a connection containing the inputs, output, and parameters.
			context.getConnectionManager().addConnection(
					new GreedyConnection(tree, log, outputTree, parameters));
		}
		// Return the output.
		Object[] result = new Object[2];
		result[1] = outputTree;
		result[0] = confProcResults;
		return result;
	}

}
